#!/usr/bin/perl
use strict;
use warnings;
use Tk;
use Tk::LabFrame;
use threads;                    # Pour créer nos threads
use threads::shared;            # Pour partager nos données entre threads
use Time::HiRes qw( sleep );    # Pour faire des sleeps < à une seconde
use Win32::API;
$Win32::API::DEBUG = 0;

my $RepBase = 'C:\Mail';
my $Fichier_Flag = File::Spec->catfile( $RepBase, 'Flag.txt' );

# Contient les fonctions à appeler dans le thread si besoin
my %fonctions_a_lancer_dans_thread = ( 'Veille' => \&Veille, );

# Invalidation de la veille d'écran
my $EcranVeille = new Win32::API("user32", "SystemParametersInfo",'II');
$EcranVeille->Call(17,0);

# Invalidation estompage et extinction écran - doit être appelé régulièrement en sub Prog_Lignes
my $EcranExtinction = new Win32::API("kernel32", "SetThreadExecutionState",'I');

#===================================
# Threads et variables partagées
#==================================
my $tuer_thread : shared;              # Permet de tuer le thread proprement
my $nom_fonction : shared;             # Contient le nom de la fonction à appeler
my $thread_travail : shared;           # Contient la valeur permettant au thread de lancer une procédure
my @arguments_thread : shared;         # Contient les arguments à passer à une éventuelle procédure
my $ref_resultat_fonction : shared;    # Contient le résultat des fonctions lancées dans le thread
my $CptTmp : shared;									 # Compteur pour mise à jour de $Cpt de label_cpt

$thread_travail = 0;                   # 0 : thread ne fait rien, 1 : il bosse
$tuer_thread    = 0;                   # 0 : thread en vie, 1 : thread se termine
$CptTmp = 1;

# Création du thread
my $thread = threads->create( \&ThreadVeille );

#===================================
# Debut du code principal Perl Tk
#==================================
my $mw = new MainWindow(
  -title      => 'Veille',
  -background => 'white',
);
$mw->overrideredirect(1);
#$mw->protocol( "WM_DELETE_WINDOW", \&fermer_application );

my $Tempo = 58;
my ( $label_etat, $label_cpt ) = ();

$mw->Button(
	-text => "Start",
	-command => sub {
		$label_etat->configure(	-background => '#00FF00',	);
  	Veille_tk ();
  },
)->pack(-side => 'left');

$mw->Button(
	-text => "Stop",
	-command => sub {
		$label_etat->configure(	-background => '#FF0000',	);
    Arret();
  },
)->pack(-side => 'left');

$mw->Button(
	-text => "Exit",
	-command => sub {
  	fermer_application ();
  },
)->pack(-side => 'left');

$label_etat  = $mw->Label(
	-width => 10,
	-textvariable => \$Tempo,
	-justify    	=> 'center',
  -background   => 'blue',
  -relief      	=> 'groove',
)->pack(-side => 'left');

my $Cpt = 0;
$label_cpt  = $mw->Label(
	-width => 8,
	-textvariable => \$Cpt,
	-justify    	=> 'center',
  -background   => '#C0C0FF',
  -relief      	=> 'groove',
)->pack(-side => 'left');

my ($Flag0, $Flag1) = ();
my $Flag = 1;
$Flag0 = $mw->Radiobutton(
		-value						=> 0,
		-background				=> '#00FF00',
		-activebackground	=> '#00FF00',
 		-variable					=> \$Flag,
 		-command					=> sub {
 			if (!open(FLAG,'>:utf8', $Fichier_Flag)) {
				print "Impossible d'ouvrir $Fichier_Flag\n";
				exit;
			}
 			$Flag = 0;
 			print FLAG $Flag;
 			close FLAG;
 			$Flag0->configure( -activebackground => '#FF0000', );
 			$Flag1->configure( -activebackground => '#FF0000', );
 			$Flag0->configure( -background => '#FF0000', );
 			$Flag1->configure( -background => '#FF0000', );
 		},
	)->pack(-side => 'left');
$Flag1 = $mw->Radiobutton(
		-value						=> 1,
		-background				=> '#00FF00',
		-activebackground	=> '#00FF00',
 		-variable					=> \$Flag,
 		-command					=> sub {
 			if (-f $Fichier_Flag) {
				unlink  $Fichier_Flag;
			}
 			$Flag = 1;
 			$Flag0->configure( -activebackground => '#00FF00', );
 			$Flag1->configure( -activebackground => '#00FF00', );
 			$Flag0->configure( -background => '#00FF00', );
 			$Flag1->configure( -background => '#00FF00', );
 		},
	)->pack(-side => 'left');

# positionner ma fenêtre
positionner_widget($mw);

# Toutes les 10 secondes, mise à jour de la fenêtre
$mw->repeat( 10000, sub { $Cpt = $CptTmp; } );

MainLoop;

#================================================
# notre_processus_leger
#================================================
sub ThreadVeille {

  # Tourne en rond
  while (1) {
    # demande au thread de travailler
    if ( $thread_travail == 1 ) {

      # Lance la procédure
      my @resultat = $fonctions_a_lancer_dans_thread{$nom_fonction}->(@arguments_thread);
      $ref_resultat_fonction = shared_clone( \@resultat );

      # demande au thread de dormir
      $thread_travail = 0;
    }

    # Terminer le thread
    last if ( $tuer_thread == 1 );
    sleep 58; # N'ayant rien à afficher dans la fenêtre initiale qui ne contient que Stop Start Exit
  }
  return;
}

sub fermer_application {

  # Demande au thread de se terminer
  $tuer_thread = 1;

  # On attend que le thread se termine proprement
  $thread->detach();
  
  $EcranVeille->Call(17,1);							# Validation de l'écran de veille
	$EcranExtinction->Call(2147483648);		# Validation estompage et extinction de l'écran + mise en veille du PC 
																				# ES_CONTINUOUS = 0x80000000 = 2147483648
	if (-f $Fichier_Flag) {
		unlink  $Fichier_Flag;
	}
  exit;
}

sub Veille_tk {
  if ( not defined $Tempo ) {
    return;
  }

  # On lui donne les arguments
  @arguments_thread = ( $Tempo );
  
  # On lui indique la procédure à appeler
  $nom_fonction = "Veille";

  # On va demander au thread de bosser
  $thread_travail = 1;
  return;
}

sub Veille {
  my ( $Tempo ) = @_;

	while (1) {
		if ( $thread_travail == 1 ) {
			$EcranExtinction->Call(2);		# Invalidation estompage et extinction écran	: ES_DISPLAY_REQUIRED = 0x00000002 
			$EcranExtinction->Call(1);		# Invalidation mise en veille PC 							: ES_SYSTEM_REQUIRED  = 0x00000001 
   		sleep $Tempo;
   		$CptTmp++;
   	} else {
   		sleep $Tempo;
   	}
	}
}

sub Arret {
	# Validation Ecran de veille, estompage et veille
	$EcranVeille->Call(17,1);
	$EcranExtinction->Call(2147483648);
	$thread_travail = 0;
}

#================================================
# Positionner la fenêtre automatiquement
#================================================
sub positionner_widget {
  my ($widget) = @_;

  # Height and width of the screen
  my $largeur_ecran = $widget->screenwidth();
  my $hauteur_ecran = $widget->screenheight();

  # update le widget pour récupérer les vraies dimensions
  $widget->update;
  my $largeur_widget = $widget->width;
  my $hauteur_widget = $widget->height;

  # On centre le widget en fonction de la taille de l'écran
  my $nouvelle_largeur = int( ( $largeur_ecran - $largeur_widget ) / 2 ); 	# Pas utilisé
  my $nouvelle_hauteur = int( ( $hauteur_ecran - ($hauteur_widget + 40)) ); # 75 c'est pour la barre des tâches
  $widget->geometry( $largeur_widget . "x" . $hauteur_widget . "+0+$nouvelle_hauteur" );

  $widget->update;

  return;
}

